struct IterNatPair {
	next: (usize, usize),
}
impl IterNatPair {
	pub fn new() -> Self {
		Self { next: (0, 0) }
	}
}
impl Iterator for IterNatPair {
	type Item = (usize, usize);
	fn next(&mut self) -> Option<Self::Item> {
		let next = self.next;
		self.next = match self.next.0 {
			0 => (self.next.1 + 1, 0),
			_ => (self.next.0 - 1, self.next.1 + 1),
		};
		Some(next)
	}
}

// TODO: If needle isn't in haystack, return None. Currently, we get stuck.
//       I think this is hard to do without breaking infinite haystacks.

// I'd be interested what time complexity this is, and how you worked it out.

pub fn divine_intellect<T: PartialEq, I: Iterator<Item = T> + Clone>(
	haystack: I,
	needle: I,
) -> impl Iterator<Item = (usize, usize)> {
	IterNatPair::new()
		.filter(|(_, step)| *step != 0)
		.filter(move |(start, step)| {
			haystack
				.clone()
				.skip(*start)
				.step_by(*step)
				.take(needle.clone().count())
				.eq(needle.clone())
		})
}

pub fn entry(haystack: &str, needle: &str) -> Option<(usize, usize)> {
	divine_intellect(haystack.chars(), needle.chars()).next()
}

#[cfg(test)]
mod tests {
	use super::*;

	#[test]
	fn iternatpair() {
		assert_eq!(
			IterNatPair::new().take(6).collect::<Vec<_>>(),
			[(0, 0), (1, 0), (0, 1), (2, 0), (1, 1), (0, 2)],
		);
	}

	#[test]
	fn short() {
		assert_eq!(entry("hello", "hello"), Some((0, 1)));
		assert_eq!(entry(" hello", "hello"), Some((1, 1)));
		assert_eq!(entry("h e l l o", "hello"), Some((0, 2)));
		assert_eq!(entry(" h e l l o", "hello"), Some((1, 2)));
		assert!(match entry("abc", "b") {
			Some((start, _)) => start == 1,
			None => false,
		});
		assert!(match entry("abc", "") {
			Some((start, _)) => start <= 3,
			None => false,
		});
		matches!(entry("", ""), Some((0, _)));
	}

	#[test]
	fn wrong_path() {
		assert_eq!(entry("ehello", "hello"), Some((1, 1)));
		assert_eq!(entry("hhello", "hello"), Some((1, 1)));
		assert_eq!(entry("he hello", "hello"), Some((3, 1)));
	}

	#[test]
	fn failure() {
		assert_eq!(entry("hello", "hi"), None);
	}

	#[test]
	fn long() {
		assert_eq!(
			entry("abhegggggggggaachjkhfasd h e l l o a", "hello"),
			Some((25, 2)),
		);
	}
}
